@@ -16,6 +16,7 @@ use crate::input::{Input, InputType, StringMapping};
 use crate::py_gc::PyGcTraverse;
 use crate::recursion_guard::RecursionState;
 use crate::tools::SchemaDict;
+pub(crate) use config::ValBytesMode;
 
 mod any;
 mod arguments;
@@ -24,6 +25,8 @@ mod bytes;
 mod call;
 mod callable;
 mod chain;
+pub(crate) mod complex;
+mod config;
 mod custom_error;
 mod dataclass;
 mod date;
@@ -117,6 +120,7 @@ pub struct SchemaValidator {
 #[pymethods]
 impl SchemaValidator {
     #[new]
+    #[pyo3(signature = (schema, config=None))]
     pub fn py_new(py: Python, schema: &Bound<'_, PyAny>, config: Option<&Bound<'_, PyDict>>) -> PyResult<Self> {
         let mut definitions_builder = DefinitionsBuilder::new();
 
@@ -475,6 +479,7 @@ macro_rules! validator_match {
             $(
                 <$validator>::EXPECTED_TYPE => build_specific_validator::<$validator>($type, $dict, $config, $definitions),
             )+
+            "invalid" => return py_schema_err!("Cannot construct schema with `InvalidSchema` member."),
             _ => return py_schema_err!(r#"Unknown schema type: "{}""#, $type),
         }
     };
@@ -579,6 +584,7 @@ pub fn build_validator(
         // recursive (self-referencing) models
         definitions::DefinitionRefValidator,
         definitions::DefinitionsValidatorBuilder,
+        complex::ComplexValidator,
     )
 }
 
@@ -732,6 +738,7 @@ pub enum CombinedValidator {
     DefinitionRef(definitions::DefinitionRefValidator),
     // input dependent
     JsonOrPython(json_or_python::JsonOrPython),
+    Complex(complex::ComplexValidator),
 }
 
 /// This trait must be implemented by all validators, it allows various validators to be accessed consistently,
