@@ -1,7 +1,8 @@
 use pyo3::intern;
-use pyo3::once_cell::GILOnceCell;
 use pyo3::prelude::*;
+use pyo3::sync::GILOnceCell;
 use pyo3::types::PyDict;
+use pyo3::types::PyString;
 use pyo3::PyTraverseError;
 use pyo3::PyVisit;
 
@@ -17,18 +18,18 @@ use crate::PydanticUndefinedType;
 static COPY_DEEPCOPY: GILOnceCell<PyObject> = GILOnceCell::new();
 
 fn get_deepcopy(py: Python) -> PyResult<PyObject> {
-    Ok(py.import("copy")?.getattr("deepcopy")?.into_py(py))
+    Ok(py.import_bound("copy")?.getattr("deepcopy")?.into_py(py))
 }
 
 #[derive(Debug, Clone)]
 pub enum DefaultType {
     None,
     Default(PyObject),
-    DefaultFactory(PyObject),
+    DefaultFactory(PyObject, bool),
 }
 
 impl DefaultType {
-    pub fn new(schema: &PyDict) -> PyResult<Self> {
+    pub fn new(schema: &Bound<'_, PyDict>) -> PyResult<Self> {
         let py = schema.py();
         match (
             schema.get_as(intern!(py, "default"))?,
@@ -36,15 +37,32 @@ impl DefaultType {
         ) {
             (Some(_), Some(_)) => py_schema_err!("'default' and 'default_factory' cannot be used together"),
             (Some(default), None) => Ok(Self::Default(default)),
-            (None, Some(default_factory)) => Ok(Self::DefaultFactory(default_factory)),
+            (None, Some(default_factory)) => Ok(Self::DefaultFactory(
+                default_factory,
+                schema
+                    .get_as::<bool>(intern!(py, "default_factory_takes_data"))?
+                    .unwrap_or(false),
+            )),
             (None, None) => Ok(Self::None),
         }
     }
 
-    pub fn default_value(&self, py: Python) -> PyResult<Option<PyObject>> {
+    pub fn default_value(&self, py: Python, validated_data: &Option<Bound<PyDict>>) -> PyResult<Option<PyObject>> {
         match self {
             Self::Default(ref default) => Ok(Some(default.clone_ref(py))),
-            Self::DefaultFactory(ref default_factory) => Ok(Some(default_factory.call0(py)?)),
+            Self::DefaultFactory(ref default_factory, ref takes_data) => {
+                let result = if *takes_data {
+                    if validated_data.is_none() {
+                        default_factory.call1(py, ({},))
+                    } else {
+                        default_factory.call1(py, (validated_data.as_deref().unwrap(),))
+                    }
+                } else {
+                    default_factory.call0(py)
+                };
+
+                Ok(Some(result?))
+            }
             Self::None => Ok(None),
         }
     }
@@ -52,7 +70,7 @@ impl DefaultType {
 
 impl PyGcTraverse for DefaultType {
     fn py_gc_traverse(&self, visit: &PyVisit<'_>) -> Result<(), PyTraverseError> {
-        if let Self::Default(obj) | Self::DefaultFactory(obj) = self {
+        if let Self::Default(obj) | Self::DefaultFactory(obj, _) = self {
             visit.call(obj)?;
         }
         Ok(())
@@ -81,13 +99,18 @@ impl BuildValidator for WithDefaultValidator {
     const EXPECTED_TYPE: &'static str = "default";
 
     fn build(
-        schema: &PyDict,
-        config: Option<&PyDict>,
+        schema: &Bound<'_, PyDict>,
+        config: Option<&Bound<'_, PyDict>>,
         definitions: &mut DefinitionsBuilder<CombinedValidator>,
     ) -> PyResult<CombinedValidator> {
         let py = schema.py();
         let default = DefaultType::new(schema)?;
-        let on_error = match schema.get_as::<&str>(intern!(py, "on_error"))? {
+        let on_error = match schema
+            .get_as::<Bound<'_, PyString>>(intern!(py, "on_error"))?
+            .as_ref()
+            .map(|s| s.to_str())
+            .transpose()?
+        {
             Some("raise") => OnError::Raise,
             Some("omit") => OnError::Omit,
             Some("default") => {
@@ -101,11 +124,11 @@ impl BuildValidator for WithDefaultValidator {
             _ => unreachable!(),
         };
 
-        let sub_schema: &PyAny = schema.get_as_req(intern!(schema.py(), "schema"))?;
-        let validator = Box::new(build_validator(sub_schema, config, definitions)?);
+        let sub_schema = schema.get_as_req(intern!(schema.py(), "schema"))?;
+        let validator = Box::new(build_validator(&sub_schema, config, definitions)?);
 
         let copy_default = if let DefaultType::Default(default_obj) = &default {
-            default_obj.as_ref(py).hash().is_err()
+            default_obj.bind(py).hash().is_err()
         } else {
             false
         };
@@ -128,11 +151,11 @@ impl BuildValidator for WithDefaultValidator {
 impl_py_gc_traverse!(WithDefaultValidator { default, validator });
 
 impl Validator for WithDefaultValidator {
-    fn validate<'data>(
+    fn validate<'py>(
         &self,
-        py: Python<'data>,
-        input: &'data impl Input<'data>,
-        state: &mut ValidationState,
+        py: Python<'py>,
+        input: &(impl Input<'py> + ?Sized),
+        state: &mut ValidationState<'_, 'py>,
     ) -> ValResult<PyObject> {
         if input.to_object(py).is(&self.undefined) {
             Ok(self.default_value(py, None::<usize>, state)?.unwrap())
@@ -151,13 +174,13 @@ impl Validator for WithDefaultValidator {
         }
     }
 
-    fn default_value(
+    fn default_value<'py>(
         &self,
-        py: Python<'_>,
+        py: Python<'py>,
         outer_loc: Option<impl Into<LocItem>>,
-        state: &mut ValidationState,
+        state: &mut ValidationState<'_, 'py>,
     ) -> ValResult<Option<PyObject>> {
-        match self.default.default_value(py)? {
+        match self.default.default_value(py, &state.extra().data)? {
             Some(stored_dft) => {
                 let dft: Py<PyAny> = if self.copy_default {
                     let deepcopy_func = COPY_DEEPCOPY.get_or_init(py, || get_deepcopy(py).unwrap());
@@ -166,11 +189,11 @@ impl Validator for WithDefaultValidator {
                     stored_dft
                 };
                 if self.validate_default {
-                    match self.validate(py, dft.into_ref(py), state) {
+                    match self.validate(py, dft.bind(py), state) {
                         Ok(v) => Ok(Some(v)),
                         Err(e) => {
                             if let Some(outer_loc) = outer_loc {
-                                Err(e.with_outer_location(outer_loc.into()))
+                                Err(e.with_outer_location(outer_loc))
                             } else {
                                 Err(e)
                             }
