@@ -1,7 +1,7 @@
 import dataclasses
 import json
-import re
 import uuid
+import warnings
 from decimal import Decimal
 from typing import Any, ClassVar, Union
 
@@ -32,9 +32,17 @@ def test_union_bool_int(input_value, expected_value, bool_case_label, int_case_l
 
 def test_union_error():
     s = SchemaSerializer(core_schema.union_schema([core_schema.bool_schema(), core_schema.int_schema()]))
-    msg = 'Expected `Union[bool, int]` but got `str` - serialized value may not be as expected'
-    with pytest.warns(UserWarning, match=re.escape(msg)):
-        assert s.to_python('a string') == 'a string'
+
+    messages = [
+        "Expected `bool` but got `str` with value `'a string'` - serialized value may not be as expected",
+        "Expected `int` but got `str` with value `'a string'` - serialized value may not be as expected",
+    ]
+
+    with warnings.catch_warnings(record=True) as w:
+        warnings.simplefilter('always')
+        s.to_python('a string')
+        for m in messages:
+            assert m in str(w[0].message)
 
 
 class ModelA:
@@ -626,3 +634,147 @@ def test_union_serializer_picks_exact_type_over_subclass_json(
     )
     assert s.to_python(input_value, mode='json') == expected_value
     assert s.to_json(input_value) == json.dumps(expected_value).encode()
+
+
+def test_tagged_union() -> None:
+    @dataclasses.dataclass
+    class ModelA:
+        field: int
+        tag: Literal['a'] = 'a'
+
+    @dataclasses.dataclass
+    class ModelB:
+        field: int
+        tag: Literal['b'] = 'b'
+
+    s = SchemaSerializer(
+        core_schema.tagged_union_schema(
+            choices={
+                'a': core_schema.dataclass_schema(
+                    ModelA,
+                    core_schema.dataclass_args_schema(
+                        'ModelA',
+                        [
+                            core_schema.dataclass_field(name='field', schema=core_schema.int_schema()),
+                            core_schema.dataclass_field(name='tag', schema=core_schema.literal_schema(['a'])),
+                        ],
+                    ),
+                    ['field', 'tag'],
+                ),
+                'b': core_schema.dataclass_schema(
+                    ModelB,
+                    core_schema.dataclass_args_schema(
+                        'ModelB',
+                        [
+                            core_schema.dataclass_field(name='field', schema=core_schema.int_schema()),
+                            core_schema.dataclass_field(name='tag', schema=core_schema.literal_schema(['b'])),
+                        ],
+                    ),
+                    ['field', 'tag'],
+                ),
+            },
+            discriminator='tag',
+        )
+    )
+
+    assert 'TaggedUnionSerializer' in repr(s)
+
+    model_a = ModelA(field=1)
+    model_b = ModelB(field=1)
+    assert s.to_python(model_a) == {'field': 1, 'tag': 'a'}
+    assert s.to_python(model_b) == {'field': 1, 'tag': 'b'}
+
+
+def test_union_float_int() -> None:
+    s = SchemaSerializer(core_schema.union_schema([core_schema.float_schema(), core_schema.int_schema()]))
+
+    assert s.to_python(1) == 1
+    assert json.loads(s.to_json(1)) == 1
+
+    s = SchemaSerializer(core_schema.union_schema([core_schema.int_schema(), core_schema.float_schema()]))
+
+    assert s.to_python(1) == 1
+    assert json.loads(s.to_json(1)) == 1
+
+
+def test_custom_serializer() -> None:
+    s = SchemaSerializer(
+        core_schema.union_schema(
+            [
+                core_schema.dict_schema(
+                    keys_schema=core_schema.any_schema(),
+                    values_schema=core_schema.any_schema(),
+                    serialization=core_schema.plain_serializer_function_ser_schema(lambda x: x['id']),
+                ),
+                core_schema.list_schema(
+                    items_schema=core_schema.dict_schema(
+                        keys_schema=core_schema.any_schema(),
+                        values_schema=core_schema.any_schema(),
+                        serialization=core_schema.plain_serializer_function_ser_schema(lambda x: x['id']),
+                    )
+                ),
+            ]
+        )
+    )
+    print(s)
+    assert s.to_python([{'id': 1}, {'id': 2}]) == [1, 2]
+    assert s.to_python({'id': 1}) == 1
+
+
+def test_tagged_union_with_aliases() -> None:
+    @dataclasses.dataclass
+    class ModelA:
+        field: int
+        tag: Literal['a'] = 'a'
+
+    @dataclasses.dataclass
+    class ModelB:
+        field: int
+        tag: Literal['b'] = 'b'
+
+    s = SchemaSerializer(
+        core_schema.tagged_union_schema(
+            choices={
+                'a': core_schema.dataclass_schema(
+                    ModelA,
+                    core_schema.dataclass_args_schema(
+                        'ModelA',
+                        [
+                            core_schema.dataclass_field(name='field', schema=core_schema.int_schema()),
+                            core_schema.dataclass_field(
+                                name='tag',
+                                schema=core_schema.literal_schema(['a']),
+                                validation_alias='TAG',
+                                serialization_alias='TAG',
+                            ),
+                        ],
+                    ),
+                    ['field', 'tag'],
+                ),
+                'b': core_schema.dataclass_schema(
+                    ModelB,
+                    core_schema.dataclass_args_schema(
+                        'ModelB',
+                        [
+                            core_schema.dataclass_field(name='field', schema=core_schema.int_schema()),
+                            core_schema.dataclass_field(
+                                name='tag',
+                                schema=core_schema.literal_schema(['b']),
+                                validation_alias='TAG',
+                                serialization_alias='TAG',
+                            ),
+                        ],
+                    ),
+                    ['field', 'tag'],
+                ),
+            },
+            discriminator=[['tag'], ['TAG']],
+        )
+    )
+
+    assert 'TaggedUnionSerializer' in repr(s)
+
+    model_a = ModelA(field=1)
+    model_b = ModelB(field=1)
+    assert s.to_python(model_a) == {'field': 1, 'TAG': 'a'}
+    assert s.to_python(model_b) == {'field': 1, 'TAG': 'b'}
