@@ -1,5 +1,8 @@
 import enum
+import os
+import pickle
 import re
+import subprocess
 import sys
 from decimal import Decimal
 from typing import Any, Optional
@@ -254,6 +257,11 @@ all_errors = [
     ('no_such_attribute', "Object has no attribute 'wrong_name'", {'attribute': 'wrong_name'}),
     ('json_invalid', 'Invalid JSON: foobar', {'error': 'foobar'}),
     ('json_type', 'JSON input should be string, bytes or bytearray', None),
+    (
+        'needs_python_object',
+        'Cannot check `isinstance` when validating from json, use a JsonOrPython validator instead',
+        {'method_name': 'isinstance'},
+    ),
     ('recursion_loop', 'Recursion error - cyclic reference detected', None),
     ('model_type', 'Input should be a valid dictionary or instance of Foobar', {'class_name': 'Foobar'}),
     ('model_attributes_type', 'Input should be a valid dictionary or object to extract fields from', None),
@@ -317,6 +325,16 @@ all_errors = [
     ('bytes_too_short', 'Data should have at least 1 byte', {'min_length': 1}),
     ('bytes_too_long', 'Data should have at most 42 bytes', {'max_length': 42}),
     ('bytes_too_long', 'Data should have at most 1 byte', {'max_length': 1}),
+    (
+        'bytes_invalid_encoding',
+        'Data should be valid base64: Invalid byte 1, offset 1',
+        {'encoding': 'base64', 'encoding_error': 'Invalid byte 1, offset 1'},
+    ),
+    (
+        'bytes_invalid_encoding',
+        'Data should be valid hex: Odd number of digits',
+        {'encoding': 'hex', 'encoding_error': 'Odd number of digits'},
+    ),
     ('value_error', 'Value error, foobar', {'error': ValueError('foobar')}),
     ('assertion_error', 'Assertion failed, foobar', {'error': AssertionError('foobar')}),
     ('literal_error', 'Input should be foo', {'expected': 'foo'}),
@@ -331,6 +349,7 @@ all_errors = [
     ('time_parsing', 'Input should be in a valid time format, foobar', {'error': 'foobar'}),
     ('datetime_type', 'Input should be a valid datetime', None),
     ('datetime_parsing', 'Input should be a valid datetime, foobar', {'error': 'foobar'}),
+    ('datetime_from_date_parsing', 'Input should be a valid datetime or date, foobar', {'error': 'foobar'}),
     ('datetime_object_invalid', 'Invalid datetime object, got foobar', {'error': 'foobar'}),
     ('datetime_past', 'Input should be in the past', None),
     ('datetime_future', 'Input should be in the future', None),
@@ -381,6 +400,16 @@ all_errors = [
         'Decimal input should have no more than 1 digit before the decimal point',
         {'whole_digits': 1},
     ),
+    (
+        'complex_type',
+        'Input should be a valid python complex object, a number, or a valid complex string following the rules at https://docs.python.org/3/library/functions.html#complex',
+        None,
+    ),
+    (
+        'complex_str_parsing',
+        'Input should be a valid complex string following the rules at https://docs.python.org/3/library/functions.html#complex',
+        None,
+    ),
 ]
 
 
@@ -393,7 +422,7 @@ def test_error_type(error_type, message, context):
 
 
 def test_all_errors_covered():
-    listed_types = set(error_type for error_type, *_ in all_errors)
+    listed_types = {error_type for error_type, *_ in all_errors}
     actual_types = {e['type'] for e in list_all_errors()}
     assert actual_types == listed_types
 
@@ -460,7 +489,7 @@ def test_custom_context_for_simple_error():
 def test_all_errors():
     errors = list_all_errors()
     # print(f'{len(errors)=}')
-    assert len(errors) == len(set(e['type'] for e in errors)), 'error types are not unique'
+    assert len(errors) == len({e['type'] for e in errors}), 'error types are not unique'
     # insert_assert(errors[:4])
     assert errors[:4] == [
         {
@@ -482,10 +511,10 @@ def test_all_errors():
             'example_context': None,
         },
         {
-            'type': 'recursion_loop',
-            'message_template_python': 'Recursion error - cyclic reference detected',
-            'example_message_python': 'Recursion error - cyclic reference detected',
-            'example_context': None,
+            'type': 'needs_python_object',
+            'message_template_python': 'Cannot check `{method_name}` when validating from json, use a JsonOrPython validator instead',
+            'example_message_python': 'Cannot check `` when validating from json, use a JsonOrPython validator instead',
+            'example_context': {'method_name': ''},
         },
     ]
 
@@ -685,7 +714,7 @@ def test_validation_error_cause_config_variants(desc: str, config: CoreConfig, e
                 s.validate_python('anything')
             assert exc_info.value.__cause__ is None
         else:
-            raise AssertionError('Unhandled result: {}'.format(expected_result))
+            raise AssertionError(f'Unhandled result: {expected_result}')
 
 
 def test_validation_error_cause_traceback_preserved():
@@ -1074,3 +1103,65 @@ def test_hide_input_in_json() -> None:
 
     for error in exc_info.value.errors(include_input=False):
         assert 'input' not in error
+
+
+@pytest.mark.skipif(
+    sys.version_info < (3, 9) and sys.implementation.name == 'pypy',
+    reason='PyPy before 3.9 cannot pickle this correctly',
+)
+def test_validation_error_pickle() -> None:
+    s = SchemaValidator({'type': 'int'})
+    with pytest.raises(ValidationError) as exc_info:
+        s.validate_python('definitely not an int')
+
+    original = exc_info.value
+    roundtripped = pickle.loads(pickle.dumps(original))
+    assert original.errors() == roundtripped.errors()
+
+
+@pytest.mark.skipif('PYDANTIC_ERRORS_INCLUDE_URL' in os.environ, reason="can't test when envvar is set")
+def test_errors_include_url() -> None:
+    s = SchemaValidator({'type': 'int'})
+    with pytest.raises(ValidationError) as exc_info:
+        s.validate_python('definitely not an int')
+    assert 'https://errors.pydantic.dev' in repr(exc_info.value)
+
+
+@pytest.mark.skipif(sys.platform == 'emscripten', reason='no subprocesses on emscripten')
+@pytest.mark.parametrize(
+    ('env_var', 'env_var_value', 'expected_to_have_url'),
+    [
+        ('PYDANTIC_ERRORS_INCLUDE_URL', None, True),
+        ('PYDANTIC_ERRORS_INCLUDE_URL', '1', True),
+        ('PYDANTIC_ERRORS_INCLUDE_URL', 'True', True),
+        ('PYDANTIC_ERRORS_INCLUDE_URL', 'no', False),
+        ('PYDANTIC_ERRORS_INCLUDE_URL', '0', False),
+        # Legacy environment variable, will raise a deprecation warning:
+        ('PYDANTIC_ERRORS_OMIT_URL', '1', False),
+        ('PYDANTIC_ERRORS_OMIT_URL', None, True),
+    ],
+)
+def test_errors_include_url_envvar(env_var, env_var_value, expected_to_have_url) -> None:
+    """
+    Test the `PYDANTIC_ERRORS_INCLUDE_URL` environment variable.
+
+    Since it can only be set before `ValidationError.__repr__()` is first called,
+    we need to spawn a subprocess to test it.
+    """
+    code = "import pydantic_core; pydantic_core.SchemaValidator({'type': 'int'}).validate_python('ooo')"
+    env = os.environ.copy()
+    env.pop('PYDANTIC_ERRORS_OMIT_URL', None)  # in case the ambient environment has it set
+    if env_var_value is not None:
+        env[env_var] = env_var_value
+    env['PYTHONDEVMODE'] = '1'  # required to surface the deprecation warning
+    result = subprocess.run(
+        [sys.executable, '-c', code],
+        stdout=subprocess.PIPE,
+        stderr=subprocess.STDOUT,
+        encoding='utf-8',
+        env=env,
+    )
+    assert result.returncode == 1
+    if 'PYDANTIC_ERRORS_OMIT_URL' in env:
+        assert 'PYDANTIC_ERRORS_OMIT_URL is deprecated' in result.stdout
+    assert ('https://errors.pydantic.dev' in result.stdout) == expected_to_have_url
